//
//  ViewController.swift
//  RealmBug
//
//  Created by Michal Rentka on 02.03.2022.
//

import UIKit

import RealmSwift

class ViewController: UIViewController {
    @IBOutlet private weak var tableView: UITableView!
    @IBOutlet private weak var statusLabel: UILabel!

    private var realm: Realm!
    private var results: Results<RItem>?
    private var snapshot: Results<RItem>?
    private var token: NotificationToken?

    override func viewDidLoad() {
        super.viewDidLoad()

        self.setupTableView()
        self.setupRealm()
        self.startObserving()
    }

    private func reload(with snapshot: Results<RItem>) {
        self.snapshot = snapshot
        self.tableView.reloadData()
    }

    @IBAction private func add() {
        guard let item = self.results?.first, item.tags.count == 1 else {
            self.statusLabel.text = "\"Tag 2\" exists"
            return
        }

        self.statusLabel.text = "Added \"Tag 2\""

        try! self.realm.write {
            let typedTag = RTypedTag()
            typedTag.type = .automatic
            typedTag.item = item
            typedTag.tag = self.realm.objects(RTag.self).filter("name == %@", "Tag 2").first
            self.realm.add(typedTag)
        }
    }

    @IBAction private func delete() {
        guard let item = self.results?.first, item.tags.count == 2,
              let tag = item.tags.filter("tag.name = %@", "Tag 2").first else {
            self.statusLabel.text = "\"Tag 2\" doesn't exist"
            return
        }

        self.statusLabel.text = "Deleted \"Tag 2\""

        try! self.realm.write {
            self.realm.delete(tag)
        }
    }

    private func startObserving() {
        let results = self.realm.objects(RItem.self).sorted(by: \.key)
        self.token = results.observe(keyPaths: ["tags"]) { [weak self] change in
            switch change {
            case .error: return
            case .initial(let results):
                self?.reload(with: results.freeze())
            case .update(let results, _, _, _):
                self?.reload(with: results.freeze())
            }
        }
        self.results = results
    }

    private func setupRealm() {
        let configuration = Realm.Configuration(inMemoryIdentifier: "TestRealm", schemaVersion: 1)
        self.realm = try! Realm(configuration: configuration)

        try! self.realm.write {
            let tag1 = RTag()
            tag1.name = "Tag 1"
            self.realm.add(tag1)

            let tag2 = RTag()
            tag2.name = "Tag 2"
            self.realm.add(tag2)

            let item1 = RItem()
            item1.key = "aaa"
            self.realm.add(item1)

            let item2 = RItem()
            item2.key = "bbb"
            self.realm.add(item2)

            let typedTag = RTypedTag()
            typedTag.type = .manual
            typedTag.item = item1
            typedTag.tag = tag1
            self.realm.add(typedTag)
        }
    }

    private func setupTableView() {
        self.tableView.register(UITableViewCell.self, forCellReuseIdentifier: "Cell")
        self.tableView.dataSource = self
    }
}

extension ViewController: UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.snapshot?.count ?? 0
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath)

        if let item = self.snapshot?[indexPath.row] {
            var configuration = cell.defaultContentConfiguration()
            configuration.text = "Item \(item.key): \(item.tags.compactMap({ $0.tag?.name }).joined(separator: "; "))"
            cell.contentConfiguration = configuration
        }

        return cell
    }
}
